# -*- encoding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _


class FeedSettings(models.Model):
    '''
    Stores Rss/Atom feed settings
    '''

    active = models.BooleanField(_('Feed enabled'), default=True)
    site_title = models.CharField(
        _('Title'),
        help_text=_('Feed title for entire site'),
        max_length=200
    )
    site_description = models.TextField(
        _('Description'),
        help_text=_('Feed description for entire site'),
        max_length=1000
    )
    latest_post_count = models.IntegerField(
        _('Post count'),
        help_text=_('How many latest Post to be displayed'),
        default=10
    )

    class Meta():
        verbose_name = _('Feed Setting')
        verbose_name_plural = _('Feed Settings')

    def __unicode__(self):

        return self.site_title



# -*- encoding: utf-8 -*-

from django.contrib import admin

from ..common.admin_actions import activate, deactivate


class LooksAdmin(admin.ModelAdmin):

    fields = ('template', 'posts_per_page', 'active')
    list_display = fields
    actions = [activate, deactivate]



# -*- encoding: utf-8 -*-

from django.utils.translation import ugettext_lazy as _

"""
Common admin actions importable from any module
"""


def activate(modeladmin, request, queryset):
    queryset.update(active=True)

activate.short_description = _('Activate selected items.')


def deactivate(modeladmin, request, queryset):
    queryset.update(active=False)

deactivate.short_description = _('Deactivate selected items.')


def enable_comments(modeladmin, request, queryset):
    queryset.update(comments=True)

enable_comments.short_description = _('Enable comments.')


def disable_comments(modeladmin, request, queryset):
    queryset.update(comments=False)

disable_comments.short_description = _('Disable comments.')



# -*- encoding: utf-8 -*-

from django import forms
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from ..common.admin_actions import activate, deactivate
from .models import Language


class LanguageAdminForm(forms.ModelForm):

    country_code = forms.RegexField(
        r'^[A-z]{2,3}$',
        label=_('Country code'),
        help_text=_('(US, UK, CZ, SK, ...)'),
        error_messages={
            'invalid': _('2-3 letter combination required (US, UK, CZ, ...)')
        }
    )

    class Meta:
        model = Language


class LanguageAdmin(admin.ModelAdmin):

    form = LanguageAdminForm
    fields = (('language', 'country_code'), 'flag', 'default', 'active')
    list_display = ('language', 'country_code', 'default', 'active')
    list_filter = ('active',)
    actions = [activate, deactivate]



# -*- encoding: utf-8 -*-

from ..common.errors import ConfigurationError
from .models import Site, Script


def get_site():
    '''
    checks for site with pk=1

    @return Site object
    @raises ConfigurationError
    '''

    try:

        site = Site.objects.get(pk=1)

        return site

    except Site.DoesNotExist:

        raise ConfigurationError('There are no Site\'s, please create one in admin')

    except Site.MultipleObjectsReturned:

        raise ConfigurationError('There is more than one site, please make sure there is exactly one, this feature may be changed in future')


def get_scripts():
    '''
    Returns all scripts from DB
    '''

    return Script.objects.all()



# -*- encoding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _


class Site(models.Model):
    """
    Model for site settings
    there is supposed to by only one Site
    (for now)
    """

    domain = models.CharField(_('Domain'), max_length=150)
    display_name = models.CharField(_('Website name'), max_length=200)
    tagline = models.CharField(
        _('Tagline'),
        help_text=_('Smaller text next to website name'),
        max_length=200,
        blank=True
    )
    footer = models.TextField(
        _('Footer'),
        help_text=_('Custom footer with HTML to be displayed on each page'),
        max_length=1000,
        blank=True
    )

    def __unicode__(self):

        return self.domain

    class Meta:
        verbose_name = _('Site')
        verbose_name_plural = _('Sites')


class Script(models.Model):
    """
    Model for scripts that should
    be inserted on each page (like footer above)

    for example google analytics/facebook/google+ ..
    """

    name = models.CharField(
        _('Name'),
        help_text=_('Script identification'),
        max_length=50
    )
    code = models.TextField(
        _('Code'),
        help_text=_('Source code for script to be inserted, including &lt;script&gt;&lt;/script&gt; tags'),
        max_length=1000
    )

    class Meta:
        verbose_name = _('Script')
        verbose_name_plural = _('Scripts')



# -*- encoding: utf-8 -*-

from django.contrib import admin


class SiteAdmin(admin.ModelAdmin):

    list_display = ('domain', 'display_name', 'tagline')

    class Media:

        js = [
            '/static/admin/js/ckeditor/ckeditor.js',
            '/static/admin/js/ckeditor/start.js',
            '/static/filebrowser/js/FB_CKEditor.js',
        ]

        css = {
            'ckeditor': ('/static/admin/js/ckeditor/contents.css',),
        }


class ScriptAdmin(admin.ModelAdmin):

    list_display = ('name', 'code')



# -*- encoding: utf-8 -*-

"""
Default URL scheme for django-pages
"""
from django.conf.urls import include, patterns, url
from django.views.generic import TemplateView

from filebrowser.sites import site

from .feed.feed import RssLatestPostsFeed, AtomLatestPostsFeed


urlpatterns = patterns(
    '',
    # filebrowser
    url(r'^admin/filebrowser/', include(site.urls)),

    # helper site (wizzard)
    url(r'^wizzard/$', TemplateView.as_view(template_name='wizzard.html')),

    # rss and atom feed
    url(r'^rss/', RssLatestPostsFeed()),
    url(r'^atom/', AtomLatestPostsFeed()),

    # robots.txt
    url(r'^robots.txt$', 'django_pages.views.robots'),
    # sitemap generator
    url(r'^sitemap.xml$', 'django_pages.views.generate_sitemap'),
    # django_pages url resolver
    url(r'^(?P<url>.*)$', 'django_pages.views.main_view'),
)



# -*- encoding: utf-8 -*-

from django.contrib import admin


class CommentAdmin(admin.ModelAdmin):

    list_display = ('id', 'comment', 'user', 'post', 'ip')
    list_display_links = ('id', 'comment')
    list_filter = ('user', 'post', 'ip')
    search_fields = ['user', 'post__title', 'ip']



# -*- encoding: utf-8 -*-

from django.contrib import admin

from ..common.admin_actions import activate, deactivate


class FeedAdmin(admin.ModelAdmin):

    list_display = ('site_title', 'latest_post_count', 'active')
    actions = [activate, deactivate]



"""
This project will give you the power of standard CMS
(content management system) like Drupal, Joomla, WordPress but with much
more speed and simplicity.

It may not have as much plugins and templates like WordPress but it is super
simple application with django admin with WYSIWYG editors for page and post
editing.

It is designed for easy web content editing with no previous experience in
HTML or CSS. Just install it once, use default template or create one for
yourself (or hire someone to make it) and use it. There is no maintenance
required.
"""

VERSION = '1.2.2'



# -*- encoding: utf-8 -*-

from django import forms

from .validators import validate_empty


class CommentForm(forms.Form):

    user = forms.CharField(max_length=100, widget=forms.TextInput(attrs={'class': 'input-xlarge'}))
    comment = forms.CharField(max_length=1000, widget=forms.Textarea(attrs={'class': 'input-xlarge'}))

    # this is dummy field to fool bots, it is hidden with CSS, if it is filled,
    # it is a bot
    email = forms.CharField(max_length=100, required=False, validators=[validate_empty])

    # this is also for spam protection, but is supposed to be filled with JS
    # required False is because we don't want django's field is required alert
    fillmeup = forms.CharField(max_length=100, required=False)

    one = forms.BooleanField(required=False)
    two = forms.BooleanField(required=False)
    three = forms.BooleanField(required=False)
    four = forms.BooleanField(required=False)



# -*- encoding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _


class Template(models.Model):
    '''
    Stores template information.
    Template name is used for template selection in templates foled of your project
    '''

    template = models.CharField(_('Template name'), max_length=200)
    posts_per_page = models.PositiveIntegerField(_('Posts per page'), default=10)
    active = models.BooleanField(_('Active'), default=False, blank=True)

    def __unicode__(self):

        return self.template

    def save(self, *args, **kwargs):
        '''

        '''

        if not self.active:
            if not Template.objects.exists():
                self.active = True

        else:
            others = Template.objects.filter(active=True).exclude(id=self.id)

            for other_active in others:
                other_active.active = False
                super(Template, other_active).save(*args, **kwargs)

            self.active = True

        super(Template, self).save(*args, **kwargs)

    class Meta:
        verbose_name = _('Template')
        verbose_name_plural = _('Templates')



# -*- encoding: utf-8 -*-

from django.utils.translation import ugettext_lazy as _

from grappelli.dashboard import modules, Dashboard
from grappelli.dashboard.utils import get_admin_site_name


class DjangoPagesDashboard(Dashboard):
    """
    Custom index dashboard for Django-pages
    """

    def init_with_context(self, context):
        site_name = get_admin_site_name(context)

        self.children.append(
            modules.ModelList(
                _('General'),
                column=1,
                collapsible=True,
                models=(
                    'django_pages.site.models.Site',
                    'django_pages.site.models.Script',
                    'django_pages.language.models.*',
                    'django_pages.looks.models.*',
                    'django_pages.feed.models.*'
                ),
            )
        )

        self.children.append(
            modules.ModelList(
                _('Pages'),
                column=1,
                collapsible=True,
                models=('django_pages.pages.models.*', )
            )
        )

        self.children.append(
            modules.ModelList(
                _('Menu'),
                column=2,
                collapsible=True,
                models=('django_pages.menu.models.*', )
            )
        )

        self.children.append(
            modules.ModelList(
                _('Comments'),
                column=2,
                collapsible=True,
                models=('django_pages.comments.models.*', )
            )
        )

        self.children.append(
            modules.ModelList(
                _('SEO'),
                column=2,
                collapsible=True,
                models=('django_pages.metadata.models.*', )
            )
        )

        self.children.append(
            modules.AppList(
                _('Administration'),
                column=1,
                collapsible=False,
                models=('django.contrib.*', )
            )
        )

        self.children.append(modules.LinkList(
            _('File Management'),
            column=3,
            children=[
                {
                    'title': _('File Browser'),
                    'url': '/admin/filebrowser/browse/',
                    'external': False,
                },
            ]
        ))

        self.children.append(modules.RecentActions(
            _('Recent Actions'),
            limit=5,
            collapsible=False,
            column=3,
        ))



# -*- encoding: utf-8 -*-

import datetime

from django.core.paginator import Paginator, EmptyPage
from django.http import Http404
from django.utils.text import slugify

from .models import Page
from ..common.errors import ConfigurationError


def get_index_page(language):
    '''
    checks for page with index=True and returns it

    @param language: Language object
    @return Page object
    '''

    try:

        page = Page.objects.select_related('link', 'metadata_set', 'link__lang').get(index=True, link__lang=language)

        return page

    except Page.DoesNotExist:

        raise ConfigurationError('There is no index Page for this Language, please make sure exactly one Page has index field checked True')

    except Page.MultipleObjectsReturned:

        raise ConfigurationError('There are multiple index Pages for this Language, please make sure only 1 Page has index field checked True')


def get_page(page_url, language, preview=False):
    '''
    returns page for specific url

    @param page_url: string
    @param language: Language object
    @return Page object
    '''

    try:

        page = Page.objects.select_related('link', 'metadata_set', 'link__lang').get(link__url=page_url, link__lang=language)

        if page.active or preview:

            return page

        else:

            raise Http404

    except Page.DoesNotExist:

        raise Http404


def get_paginated_posts(page, page_num=1, posts_on_page=10):
    '''
    returns posts paginated by settings.POSTS_ON_PAGE for page object or ()

    @param page: Page object
    @return Paginator object
    '''

    if page.have_posts():

        posts = page.post_set.filter(active=True).order_by('-created')

        posts_visible = [item for item in posts if item.is_visible(datetime.datetime.now())]

        paginated_posts = Paginator(posts_visible, posts_on_page)

        try:

            return paginated_posts.page(page_num)

        except EmptyPage:

            return paginated_posts.page(paginated_posts.num_pages)

    return tuple()


def get_post(page, post_title, preview=False):
    '''
    returns post object or 404

    @param page: Page object
    @param post_title: string
    @return Post object
    '''

    for post in page.post_set.all():

        if slugify(post.title) == post_title and (post.active or preview):

            return post

    raise Http404



# -*- encoding: utf-8 -*-

"""
This settings file is used to override your project's settings with
some defaults required to run django-pages
"""

from django.conf import settings


ADMIN_MEDIA_PREFIX = getattr(settings, 'ADMIN_MEDIA_PREFIX', '/static/admin/')

# where to upload flag images /media/FLAG_UPLOAD_DIR
FLAG_UPLOAD_DIR = getattr(settings, 'FLAG_UPLOAD_DIR', settings.MEDIA_ROOT)




# -*- encoding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _

from ..language.models import Language


class MetaSet(models.Model):
    '''
    Stores MetaData configurations
    '''

    language = models.ForeignKey(Language, verbose_name=_('Language'))
    name = models.CharField(
        _('Name'),
        help_text=_('Name of meta set (for identification).'),
        max_length=200
    )

    def __unicode__(self):

        return self.name

    class Meta:
        verbose_name = _('Meta set')
        verbose_name_plural = _('Meta sets')


class MetaData(models.Model):
    '''
    Stores MetaData item
    '''

    meta_set = models.ForeignKey(MetaSet, verbose_name=_('Meta set'))
    name = models.CharField(
        _('Name'),
        help_text=_('Name of meta tag &lt;meta name="THIS NAME" ... &gt;'),
        max_length=200
    )
    content = models.TextField(
        _('Content'),
        help_text=_('Content of meta tag &lt;meta ... content="THIS CONTENT" &gt;'),
        max_length=2000
    )

    def __unicode__(self):

        return self.name

    class Meta:
        verbose_name = _('Meta data')
        verbose_name_plural = _('Meta data')



# -*- encoding: utf-8 -*-

import random

from django.utils.html import strip_tags

from .forms import CommentForm
from .models import Comment


def get_client_ip(request):
    """
    Returns requester's IP address from HTTP request

    @param request: Http Request
    @return string
    """

    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')

    if x_forwarded_for:

        ip = x_forwarded_for.split(',')[0]

    else:

        ip = request.META.get('REMOTE_ADDR')

    return ip


def set_humanity_check(request):
    """
    sets session['humanity'] to dictionary {0: True/False, 1: True/False, .. 3: True/False}

    @param request: Http Request
    @return None
    """

    result = {}

    # this ensures we don't have empty checkboxes
    while result == {} or result == {0: False, 1: False, 2: False, 3: False}:

        for i in xrange(4):

            if random.randint(0, 1):

                result[i] = True

            else:

                result[i] = False

    request.session['humanity'] = result

    # also fill the random number into session
    # this number will be filled into form using JS
    request.session['random_number'] = str(random.random())
    print request.session['random_number']


def translate_humanity(request):
    """
    translates request.session['humanity'] dictionary {0: True, 1: False, ..}
    into 'One, Two, Three' according to numbers that are True

    @param request: Http Request
    @return string
    """

    numbers = []
    translation = {0: 'one', 1: 'two', 2: 'three', 3: 'four'}

    for i in request.session['humanity']:

        if request.session['humanity'][i]:
            numbers.append(translation[i])

    check_string = ', '.join(numbers)

    return check_string


def is_human(request, data):

    if request.session['humanity'] == data:

        return True

    return False


def handle_comment(request, post):
    '''
    handles comment and either saves it or displays error

    @param request: Http Request
    @param post: Post object
    @return None or errorfrom django.utils.timezone import make_aware, get_current_timezone
    '''

    form = CommentForm(request.POST)
    ip = get_client_ip(request)

    if form.is_valid():

        user = strip_tags(form.cleaned_data['user'])
        comment = strip_tags(form.cleaned_data['comment'])

        one = form.cleaned_data['one']
        two = form.cleaned_data['two']
        three = form.cleaned_data['three']
        four = form.cleaned_data['four']

        random_number = form.cleaned_data['fillmeup']

        if random_number != request.session['random_number']:

            form.spam = 'Please check that you have JavaScript enabled.'
            return form

        if is_human(request, {0: one, 1: two, 2: three, 3: four}):

            # check for duplicate comments
            if Comment.objects.filter(post=post, user=user, comment=comment):

                return CommentForm()

            comment = Comment(post=post, user=user, comment=comment, ip=ip)
            comment.save()

            return CommentForm()

        else:

            form.spam = 'Wrong antispam check'
            return form

    else:

        return form



# -*- encoding: utf-8 -*-

from django.contrib import admin

from .models import MetaData


class MetaDataInline(admin.TabularInline):

    model = MetaData


class MetaSetAdmin(admin.ModelAdmin):

    fields = (('language', 'name'),)
    list_display = ('name', 'language')
    inlines = (MetaDataInline, )
    list_filter = ('language', )
    search_fields = ['metadata__name', 'metadata__content']


class MetaDataAdmin(admin.ModelAdmin):

    list_display = ('name', 'content', 'meta_set')
    list_filter = ('meta_set__name', 'name')
    search_fields = ['content']



# -*- encoding: utf-8 -*-

from django.conf.urls import patterns
from django.contrib import admin
from django.core.exceptions import PermissionDenied
from django.shortcuts import get_object_or_404

from ..common.admin_actions import activate, deactivate, enable_comments, disable_comments
from .models import Page, Post
from ..views import main_view


class PreviewAdmin(admin.ModelAdmin):
    """
    Custom parent class, with preview support
    """

    readonly_fields = ['preview_link', ]

    class Media:

        js = [
            '/static/admin/js/ckeditor/ckeditor.js',
            '/static/admin/js/ckeditor/start.js',
            '/static/filebrowser/js/FB_CKEditor.js',        
            ]

        css = {
            'ckeditor': ('/static/admin/js/ckeditor/contents.css',),
        }

    def get_urls(self):

        admin_view = self.admin_site.admin_view

        urls = patterns('',
                        (r'^(?P<item_pk>\d+)/preview/$', admin_view(self.preview_view)),
                        )

        return urls + super(PreviewAdmin, self).get_urls()

    def preview_link(self, obj):
        """
        Returns url for page preview

        @return string
        """

        return '<a href="./preview/">Page preview</a>'

    preview_link.allow_tags = True
    preview_link.short_description = 'Preview'

    def preview_view(self, request, item_pk):

        pass


class PageAdmin(PreviewAdmin):

    fieldsets = (
        (None, {
            'fields': (('title', 'link'), 'content', 'active', 'index', 'preview_link')
        }),
        ('Advanced options', {
            'classes': ('collapse',),
            'fields': ('metadata_set',)
        }),
    )

    list_editable = ('link', 'active', 'metadata_set')
    list_display = ('title', 'link', 'active', 'metadata_set',)
    list_filter = ('active', 'index')
    search_fields = ('title', 'content', 'link__menuitem_name')
    actions = [activate, deactivate]

    def preview_view(self, request, item_pk):

        if self.has_change_permission(request):

            item = get_object_or_404(Page, pk=item_pk)
            item.active = False
            item.save()

            url = item.get_url()[1:]

            return main_view(request, url, preview=True)

        else:

            raise PermissionDenied


class PostAdmin(PreviewAdmin):

    fields = (('title', 'page'), 'content', 'active', 'comments', 'preview_link', 'visible_from', 'visible_to', 'created')
    list_editable = ('page', 'active', 'visible_from', 'visible_to', 'comments')
    list_display = ('title', 'page', 'active', 'visible_from', 'visible_to', 'comments')
    list_filter = ('active', 'page')
    search_fields = ['title', 'content', 'page__title']
    actions = [activate, deactivate, enable_comments, disable_comments]

    def preview_view(self, request, item_pk):

        if self.has_change_permission(request):

            item = get_object_or_404(Post, pk=item_pk)
            item.active = False
            item.save()

            url = item.get_url()[1:]

            return main_view(request, url, preview=True)

        else:

            raise PermissionDenied



#! -*- encoding: utf-8 -*-

from django.core.exceptions import ValidationError


def validate_empty(value):
    """ Validates that field is empty
    This is a protection against spam bots
    the field is supposed to be empty because
    it is hidden by CSS from the user """

    if value:
        raise ValidationError(u'Email must not be filled, you are a SPAM BOT!')
